package owg.competent;

import java.util.LinkedHashSet;

/**List of objects to be notified when the owner of the list is deleted.*/
public class CADeletionListenerList 
{
	protected final Object lock;
	protected final LinkedHashSet<CADeletionListener> listeners;
	
	/**Create a new list of deletion listeners.
	 * @param lock A lock that is used to synchronize accesses to this list.*/
	public CADeletionListenerList(Object lock) {
		this.lock = lock;
		this.listeners = new LinkedHashSet<CADeletionListener>();
	}
	
	/**Add a listener. This method is synchronized on the list's lock.*/
	public void add(CADeletionListener l)
	{
		synchronized (lock)
		{
			listeners.add(l);	
		}
	}
	/**Remove a listener. This method is synchronized on the list's lock.*/
	public void remove(CADeletionListener l)
	{
		synchronized (lock) 
		{
			listeners.remove(l);
		}
	}
	/**Fire deleted events and clear the list. This method is synchronized on the list's lock.*/
	public void deleted(Object obj)
	{
		synchronized (lock) 
		{
			for(CADeletionListener l : listeners)
				l.deleted(obj);
		}
	}
}
